package java.util;

import java.util.function.IntBinaryOperator;
import java.util.function.DoubleBinaryOperator;
import java.util.function.LongBinaryOperator;
import java.util.concurrent.ForkJoinPool;
import java.util.function.BinaryOperator;
import java.util.concurrent.CountedCompleter;

class ArrayPrefixHelpers
{
    static final int CUMULATE = 1;
    static final int SUMMED = 2;
    static final int FINISHED = 4;
    static final int MIN_PARTITION = 16;
    
    private ArrayPrefixHelpers() {
    }
    
    static final class CumulateTask<T> extends CountedCompleter<Void>
    {
        final T[] array;
        final BinaryOperator<T> function;
        ArrayPrefixHelpers.CumulateTask<T> left;
        ArrayPrefixHelpers.CumulateTask<T> right;
        T in;
        T out;
        final int lo;
        final int hi;
        final int origin;
        final int fence;
        final int threshold;
        
        public CumulateTask(ArrayPrefixHelpers.CumulateTask<T> completer, BinaryOperator<T> function, T[] array, int n, int n2) {
            super(completer);
            this.function = function;
            this.array = array;
            this.origin = n;
            this.lo = n;
            this.fence = n2;
            this.hi = n2;
            int n3;
            this.threshold = (((n3 = (n2 - n) / (ForkJoinPool.getCommonPoolParallelism() << 3)) <= 16) ? 16 : n3);
        }
        
        CumulateTask(ArrayPrefixHelpers.CumulateTask<T> completer, BinaryOperator<T> function, T[] array, int origin, int fence, int threshold, int lo, int hi) {
            super(completer);
            this.function = function;
            this.array = array;
            this.origin = origin;
            this.fence = fence;
            this.threshold = threshold;
            this.lo = lo;
            this.hi = hi;
        }
        
        @Override
        public final void compute() {
            BinaryOperator<Object> function;
            Object[] array;
            if ((function = this.function) == null || (array = this.array) == null) {
                throw new NullPointerException();
            }
            int threshold = this.threshold;
            int origin = this.origin;
            int fence = this.fence;
            ArrayPrefixHelpers.CumulateTask<Object> cumulateTask = this;
            int lo;
            int hi;
        Label_0756:
            while ((lo = cumulateTask.lo) >= 0 && (hi = cumulateTask.hi) <= array.length) {
                if (hi - lo > threshold) {
                    ArrayPrefixHelpers.CumulateTask<Object> left2 = cumulateTask.left;
                    ArrayPrefixHelpers.CumulateTask<Object> right2 = cumulateTask.right;
                    Object o;
                    Label_0322: {
                        if (left2 == null) {
                            int n4 = lo + hi >>> 1;
                            ArrayPrefixHelpers.CumulateTask right3 = new ArrayPrefixHelpers.CumulateTask(cumulateTask, function, array, origin, fence, threshold, n4, hi);
                            cumulateTask.right = right3;
                            o = right3;
                            ArrayPrefixHelpers.CumulateTask left3 = new ArrayPrefixHelpers.CumulateTask(cumulateTask, function, array, origin, fence, threshold, lo, n4);
                            cumulateTask.left = left3;
                            cumulateTask = left3;
                        }
                        else {
                            Object in = cumulateTask.in;
                            left2.in = in;
                            cumulateTask = (o = null);
                            if (right2 != null) {
                                Object out3 = left2.out;
                                right2.in = ((lo == origin) ? out3 : function.apply(in, out3));
                                int pendingCount3;
                                while (((pendingCount3 = right2.getPendingCount()) & 0x1) == 0x0) {
                                    if (!right2.compareAndSetPendingCount(pendingCount3, pendingCount3 | 0x1)) {
                                        continue;
                                    }
                                    cumulateTask = right2;
                                    break;
                                }
                            }
                            Label_0314: {
                                int pendingCount4;
                                while (((pendingCount4 = left2.getPendingCount()) & 0x1) == 0x0) {
                                    if (!left2.compareAndSetPendingCount(pendingCount4, pendingCount4 | 0x1)) {
                                        continue;
                                    }
                                    if (cumulateTask != null) {
                                        o = cumulateTask;
                                    }
                                    cumulateTask = left2;
                                    if (cumulateTask == null) {
                                        break Label_0756;
                                    }
                                    break Label_0322;
                                }
                                continue;
                            }
                        }
                    }
                    if (o != null) {
                        o.fork();
                    }
                    else {
                        continue;
                    }
                }
                else {
                    int pendingCount;
                    while (((pendingCount = cumulateTask.getPendingCount()) & 0x4) == 0x0) {
                        int n = ((pendingCount & 0x1) != 0x0) ? 4 : ((lo > origin) ? 2 : 6);
                        if (!cumulateTask.compareAndSetPendingCount(pendingCount, pendingCount | n)) {
                            continue;
                        }
                        Object out;
                        if (n != 2) {
                            int n2;
                            if (lo == origin) {
                                out = array[origin];
                                n2 = origin + 1;
                            }
                            else {
                                out = cumulateTask.in;
                                n2 = lo;
                            }
                            for (int j = n2; j < hi; ++j) {
                                out = (array[j] = function.apply(out, array[j]));
                            }
                        }
                        else if (hi < fence) {
                            out = array[lo];
                            for (int i = lo + 1; i < hi; ++i) {
                                out = function.apply(out, array[i]);
                            }
                        }
                        else {
                            out = cumulateTask.in;
                        }
                        cumulateTask.out = out;
                        ArrayPrefixHelpers.CumulateTask<Object> cumulateTask2;
                        while ((cumulateTask2 = (ArrayPrefixHelpers.CumulateTask)cumulateTask.getCompleter()) != null) {
                            int pendingCount2 = cumulateTask2.getPendingCount();
                            if ((pendingCount2 & n & 0x4) != 0x0) {
                                cumulateTask = cumulateTask2;
                            }
                            else if ((pendingCount2 & n & 0x2) != 0x0) {
                                ArrayPrefixHelpers.CumulateTask<Object> left;
                                ArrayPrefixHelpers.CumulateTask<Object> right;
                                if ((left = cumulateTask2.left) != null && (right = cumulateTask2.right) != null) {
                                    Object out2 = left.out;
                                    cumulateTask2.out = ((right.hi == fence) ? out2 : function.apply(out2, right.out));
                                }
                                int n3 = (pendingCount2 & 0x1) == 0x0 && cumulateTask2.lo == origin;
                                int count;
                                if ((count = (pendingCount2 | n | n3)) == pendingCount2 || cumulateTask2.compareAndSetPendingCount(pendingCount2, count)) {
                                    n = 2;
                                    cumulateTask = cumulateTask2;
                                    if (n3 != 0) {
                                        cumulateTask2.fork();
                                    }
                                    else {
                                        continue;
                                    }
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                if (cumulateTask2.compareAndSetPendingCount(pendingCount2, pendingCount2 | n)) {
                                    break Label_0756;
                                }
                                continue;
                            }
                        }
                        if ((n & 0x4) != 0x0) {
                            cumulateTask.quietlyComplete();
                            break;
                        }
                        break;
                    }
                    break;
                }
            }
            return;
        }
    }
    
    static final class LongCumulateTask extends CountedCompleter<Void>
    {
        final long[] array;
        final LongBinaryOperator function;
        ArrayPrefixHelpers.LongCumulateTask left;
        ArrayPrefixHelpers.LongCumulateTask right;
        long in;
        long out;
        final int lo;
        final int hi;
        final int origin;
        final int fence;
        final int threshold;
        
        public LongCumulateTask(ArrayPrefixHelpers.LongCumulateTask completer, LongBinaryOperator function, long[] array, int n, int n2) {
            super(completer);
            this.function = function;
            this.array = array;
            this.origin = n;
            this.lo = n;
            this.fence = n2;
            this.hi = n2;
            int n3;
            this.threshold = (((n3 = (n2 - n) / (ForkJoinPool.getCommonPoolParallelism() << 3)) <= 16) ? 16 : n3);
        }
        
        LongCumulateTask(ArrayPrefixHelpers.LongCumulateTask completer, LongBinaryOperator function, long[] array, int origin, int fence, int threshold, int lo, int hi) {
            super(completer);
            this.function = function;
            this.array = array;
            this.origin = origin;
            this.fence = fence;
            this.threshold = threshold;
            this.lo = lo;
            this.hi = hi;
        }
        
        @Override
        public final void compute() {
            LongBinaryOperator function;
            long[] array;
            if ((function = this.function) == null || (array = this.array) == null) {
                throw new NullPointerException();
            }
            int threshold = this.threshold;
            int origin = this.origin;
            int fence = this.fence;
            ArrayPrefixHelpers.LongCumulateTask longCumulateTask = this;
            int lo;
            int hi;
        Label_0756:
            while ((lo = longCumulateTask.lo) >= 0 && (hi = longCumulateTask.hi) <= array.length) {
                if (hi - lo > threshold) {
                    ArrayPrefixHelpers.LongCumulateTask left2 = longCumulateTask.left;
                    ArrayPrefixHelpers.LongCumulateTask right2 = longCumulateTask.right;
                    Object o;
                    Label_0322: {
                        if (left2 == null) {
                            int n3 = lo + hi >>> 1;
                            ArrayPrefixHelpers.LongCumulateTask right3 = new ArrayPrefixHelpers.LongCumulateTask(longCumulateTask, function, array, origin, fence, threshold, n3, hi);
                            longCumulateTask.right = right3;
                            o = right3;
                            ArrayPrefixHelpers.LongCumulateTask left3 = new ArrayPrefixHelpers.LongCumulateTask(longCumulateTask, function, array, origin, fence, threshold, lo, n3);
                            longCumulateTask.left = left3;
                            longCumulateTask = left3;
                        }
                        else {
                            long in = longCumulateTask.in;
                            left2.in = in;
                            longCumulateTask = (o = null);
                            if (right2 != null) {
                                long out3 = left2.out;
                                right2.in = ((lo == origin) ? out3 : function.applyAsLong(in, out3));
                                int pendingCount3;
                                while (((pendingCount3 = right2.getPendingCount()) & 0x1) == 0x0) {
                                    if (!right2.compareAndSetPendingCount(pendingCount3, pendingCount3 | 0x1)) {
                                        continue;
                                    }
                                    longCumulateTask = right2;
                                    break;
                                }
                            }
                            Label_0314: {
                                int pendingCount4;
                                while (((pendingCount4 = left2.getPendingCount()) & 0x1) == 0x0) {
                                    if (!left2.compareAndSetPendingCount(pendingCount4, pendingCount4 | 0x1)) {
                                        continue;
                                    }
                                    if (longCumulateTask != null) {
                                        o = longCumulateTask;
                                    }
                                    longCumulateTask = left2;
                                    if (longCumulateTask == null) {
                                        break Label_0756;
                                    }
                                    break Label_0322;
                                }
                                continue;
                            }
                        }
                    }
                    if (o != null) {
                        o.fork();
                    }
                    else {
                        continue;
                    }
                }
                else {
                    int pendingCount;
                    while (((pendingCount = longCumulateTask.getPendingCount()) & 0x4) == 0x0) {
                        int n = ((pendingCount & 0x1) != 0x0) ? 4 : ((lo > origin) ? 2 : 6);
                        if (!longCumulateTask.compareAndSetPendingCount(pendingCount, pendingCount | n)) {
                            continue;
                        }
                        long out;
                        if (n != 2) {
                            int j;
                            if (lo == origin) {
                                out = array[origin];
                                j = origin + 1;
                            }
                            else {
                                out = longCumulateTask.in;
                                j = lo;
                            }
                            for (; j < hi; ++j) {
                                out = (array[j] = function.applyAsLong(out, array[j]));
                            }
                        }
                        else if (hi < fence) {
                            out = array[lo];
                            for (int i = lo + 1; i < hi; ++i) {
                                out = function.applyAsLong(out, array[i]);
                            }
                        }
                        else {
                            out = longCumulateTask.in;
                        }
                        longCumulateTask.out = out;
                        ArrayPrefixHelpers.LongCumulateTask longCumulateTask2;
                        while ((longCumulateTask2 = (ArrayPrefixHelpers.LongCumulateTask)longCumulateTask.getCompleter()) != null) {
                            int pendingCount2 = longCumulateTask2.getPendingCount();
                            if ((pendingCount2 & n & 0x4) != 0x0) {
                                longCumulateTask = longCumulateTask2;
                            }
                            else if ((pendingCount2 & n & 0x2) != 0x0) {
                                ArrayPrefixHelpers.LongCumulateTask left;
                                ArrayPrefixHelpers.LongCumulateTask right;
                                if ((left = longCumulateTask2.left) != null && (right = longCumulateTask2.right) != null) {
                                    long out2 = left.out;
                                    longCumulateTask2.out = ((right.hi == fence) ? out2 : function.applyAsLong(out2, right.out));
                                }
                                int n2 = (pendingCount2 & 0x1) == 0x0 && longCumulateTask2.lo == origin;
                                int count;
                                if ((count = (pendingCount2 | n | n2)) == pendingCount2 || longCumulateTask2.compareAndSetPendingCount(pendingCount2, count)) {
                                    n = 2;
                                    longCumulateTask = longCumulateTask2;
                                    if (n2 != 0) {
                                        longCumulateTask2.fork();
                                    }
                                    else {
                                        continue;
                                    }
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                if (longCumulateTask2.compareAndSetPendingCount(pendingCount2, pendingCount2 | n)) {
                                    break Label_0756;
                                }
                                continue;
                            }
                        }
                        if ((n & 0x4) != 0x0) {
                            longCumulateTask.quietlyComplete();
                            break;
                        }
                        break;
                    }
                    break;
                }
            }
            return;
        }
    }
    
    static final class DoubleCumulateTask extends CountedCompleter<Void>
    {
        final double[] array;
        final DoubleBinaryOperator function;
        ArrayPrefixHelpers.DoubleCumulateTask left;
        ArrayPrefixHelpers.DoubleCumulateTask right;
        double in;
        double out;
        final int lo;
        final int hi;
        final int origin;
        final int fence;
        final int threshold;
        
        public DoubleCumulateTask(ArrayPrefixHelpers.DoubleCumulateTask completer, DoubleBinaryOperator function, double[] array, int n, int n2) {
            super(completer);
            this.function = function;
            this.array = array;
            this.origin = n;
            this.lo = n;
            this.fence = n2;
            this.hi = n2;
            int n3;
            this.threshold = (((n3 = (n2 - n) / (ForkJoinPool.getCommonPoolParallelism() << 3)) <= 16) ? 16 : n3);
        }
        
        DoubleCumulateTask(ArrayPrefixHelpers.DoubleCumulateTask completer, DoubleBinaryOperator function, double[] array, int origin, int fence, int threshold, int lo, int hi) {
            super(completer);
            this.function = function;
            this.array = array;
            this.origin = origin;
            this.fence = fence;
            this.threshold = threshold;
            this.lo = lo;
            this.hi = hi;
        }
        
        @Override
        public final void compute() {
            DoubleBinaryOperator function;
            double[] array;
            if ((function = this.function) == null || (array = this.array) == null) {
                throw new NullPointerException();
            }
            int threshold = this.threshold;
            int origin = this.origin;
            int fence = this.fence;
            ArrayPrefixHelpers.DoubleCumulateTask doubleCumulateTask = this;
            int lo;
            int hi;
        Label_0756:
            while ((lo = doubleCumulateTask.lo) >= 0 && (hi = doubleCumulateTask.hi) <= array.length) {
                if (hi - lo > threshold) {
                    ArrayPrefixHelpers.DoubleCumulateTask left2 = doubleCumulateTask.left;
                    ArrayPrefixHelpers.DoubleCumulateTask right2 = doubleCumulateTask.right;
                    Object o;
                    Label_0322: {
                        if (left2 == null) {
                            int n3 = lo + hi >>> 1;
                            ArrayPrefixHelpers.DoubleCumulateTask right3 = new ArrayPrefixHelpers.DoubleCumulateTask(doubleCumulateTask, function, array, origin, fence, threshold, n3, hi);
                            doubleCumulateTask.right = right3;
                            o = right3;
                            ArrayPrefixHelpers.DoubleCumulateTask left3 = new ArrayPrefixHelpers.DoubleCumulateTask(doubleCumulateTask, function, array, origin, fence, threshold, lo, n3);
                            doubleCumulateTask.left = left3;
                            doubleCumulateTask = left3;
                        }
                        else {
                            double in = doubleCumulateTask.in;
                            left2.in = in;
                            doubleCumulateTask = (o = null);
                            if (right2 != null) {
                                double out3 = left2.out;
                                right2.in = ((lo == origin) ? out3 : function.applyAsDouble(in, out3));
                                int pendingCount3;
                                while (((pendingCount3 = right2.getPendingCount()) & 0x1) == 0x0) {
                                    if (!right2.compareAndSetPendingCount(pendingCount3, pendingCount3 | 0x1)) {
                                        continue;
                                    }
                                    doubleCumulateTask = right2;
                                    break;
                                }
                            }
                            Label_0314: {
                                int pendingCount4;
                                while (((pendingCount4 = left2.getPendingCount()) & 0x1) == 0x0) {
                                    if (!left2.compareAndSetPendingCount(pendingCount4, pendingCount4 | 0x1)) {
                                        continue;
                                    }
                                    if (doubleCumulateTask != null) {
                                        o = doubleCumulateTask;
                                    }
                                    doubleCumulateTask = left2;
                                    if (doubleCumulateTask == null) {
                                        break Label_0756;
                                    }
                                    break Label_0322;
                                }
                                continue;
                            }
                        }
                    }
                    if (o != null) {
                        o.fork();
                    }
                    else {
                        continue;
                    }
                }
                else {
                    int pendingCount;
                    while (((pendingCount = doubleCumulateTask.getPendingCount()) & 0x4) == 0x0) {
                        int n = ((pendingCount & 0x1) != 0x0) ? 4 : ((lo > origin) ? 2 : 6);
                        if (!doubleCumulateTask.compareAndSetPendingCount(pendingCount, pendingCount | n)) {
                            continue;
                        }
                        double out;
                        if (n != 2) {
                            int j;
                            if (lo == origin) {
                                out = array[origin];
                                j = origin + 1;
                            }
                            else {
                                out = doubleCumulateTask.in;
                                j = lo;
                            }
                            for (; j < hi; ++j) {
                                out = (array[j] = function.applyAsDouble(out, array[j]));
                            }
                        }
                        else if (hi < fence) {
                            out = array[lo];
                            for (int i = lo + 1; i < hi; ++i) {
                                out = function.applyAsDouble(out, array[i]);
                            }
                        }
                        else {
                            out = doubleCumulateTask.in;
                        }
                        doubleCumulateTask.out = out;
                        ArrayPrefixHelpers.DoubleCumulateTask doubleCumulateTask2;
                        while ((doubleCumulateTask2 = (ArrayPrefixHelpers.DoubleCumulateTask)doubleCumulateTask.getCompleter()) != null) {
                            int pendingCount2 = doubleCumulateTask2.getPendingCount();
                            if ((pendingCount2 & n & 0x4) != 0x0) {
                                doubleCumulateTask = doubleCumulateTask2;
                            }
                            else if ((pendingCount2 & n & 0x2) != 0x0) {
                                ArrayPrefixHelpers.DoubleCumulateTask left;
                                ArrayPrefixHelpers.DoubleCumulateTask right;
                                if ((left = doubleCumulateTask2.left) != null && (right = doubleCumulateTask2.right) != null) {
                                    double out2 = left.out;
                                    doubleCumulateTask2.out = ((right.hi == fence) ? out2 : function.applyAsDouble(out2, right.out));
                                }
                                int n2 = (pendingCount2 & 0x1) == 0x0 && doubleCumulateTask2.lo == origin;
                                int count;
                                if ((count = (pendingCount2 | n | n2)) == pendingCount2 || doubleCumulateTask2.compareAndSetPendingCount(pendingCount2, count)) {
                                    n = 2;
                                    doubleCumulateTask = doubleCumulateTask2;
                                    if (n2 != 0) {
                                        doubleCumulateTask2.fork();
                                    }
                                    else {
                                        continue;
                                    }
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                if (doubleCumulateTask2.compareAndSetPendingCount(pendingCount2, pendingCount2 | n)) {
                                    break Label_0756;
                                }
                                continue;
                            }
                        }
                        if ((n & 0x4) != 0x0) {
                            doubleCumulateTask.quietlyComplete();
                            break;
                        }
                        break;
                    }
                    break;
                }
            }
            return;
        }
    }
    
    static final class IntCumulateTask extends CountedCompleter<Void>
    {
        final int[] array;
        final IntBinaryOperator function;
        ArrayPrefixHelpers.IntCumulateTask left;
        ArrayPrefixHelpers.IntCumulateTask right;
        int in;
        int out;
        final int lo;
        final int hi;
        final int origin;
        final int fence;
        final int threshold;
        
        public IntCumulateTask(ArrayPrefixHelpers.IntCumulateTask completer, IntBinaryOperator function, int[] array, int n, int n2) {
            super(completer);
            this.function = function;
            this.array = array;
            this.origin = n;
            this.lo = n;
            this.fence = n2;
            this.hi = n2;
            int n3;
            this.threshold = (((n3 = (n2 - n) / (ForkJoinPool.getCommonPoolParallelism() << 3)) <= 16) ? 16 : n3);
        }
        
        IntCumulateTask(ArrayPrefixHelpers.IntCumulateTask completer, IntBinaryOperator function, int[] array, int origin, int fence, int threshold, int lo, int hi) {
            super(completer);
            this.function = function;
            this.array = array;
            this.origin = origin;
            this.fence = fence;
            this.threshold = threshold;
            this.lo = lo;
            this.hi = hi;
        }
        
        @Override
        public final void compute() {
            IntBinaryOperator function;
            int[] array;
            if ((function = this.function) == null || (array = this.array) == null) {
                throw new NullPointerException();
            }
            int threshold = this.threshold;
            int origin = this.origin;
            int fence = this.fence;
            ArrayPrefixHelpers.IntCumulateTask intCumulateTask = this;
            int lo;
            int hi;
        Label_0756:
            while ((lo = intCumulateTask.lo) >= 0 && (hi = intCumulateTask.hi) <= array.length) {
                if (hi - lo > threshold) {
                    ArrayPrefixHelpers.IntCumulateTask left2 = intCumulateTask.left;
                    ArrayPrefixHelpers.IntCumulateTask right2 = intCumulateTask.right;
                    Object o;
                    Label_0322: {
                        if (left2 == null) {
                            int n4 = lo + hi >>> 1;
                            ArrayPrefixHelpers.IntCumulateTask right3 = new ArrayPrefixHelpers.IntCumulateTask(intCumulateTask, function, array, origin, fence, threshold, n4, hi);
                            intCumulateTask.right = right3;
                            o = right3;
                            ArrayPrefixHelpers.IntCumulateTask left3 = new ArrayPrefixHelpers.IntCumulateTask(intCumulateTask, function, array, origin, fence, threshold, lo, n4);
                            intCumulateTask.left = left3;
                            intCumulateTask = left3;
                        }
                        else {
                            int in = intCumulateTask.in;
                            left2.in = in;
                            intCumulateTask = (o = null);
                            if (right2 != null) {
                                int out3 = left2.out;
                                right2.in = ((lo == origin) ? out3 : function.applyAsInt(in, out3));
                                int pendingCount3;
                                while (((pendingCount3 = right2.getPendingCount()) & 0x1) == 0x0) {
                                    if (!right2.compareAndSetPendingCount(pendingCount3, pendingCount3 | 0x1)) {
                                        continue;
                                    }
                                    intCumulateTask = right2;
                                    break;
                                }
                            }
                            Label_0314: {
                                int pendingCount4;
                                while (((pendingCount4 = left2.getPendingCount()) & 0x1) == 0x0) {
                                    if (!left2.compareAndSetPendingCount(pendingCount4, pendingCount4 | 0x1)) {
                                        continue;
                                    }
                                    if (intCumulateTask != null) {
                                        o = intCumulateTask;
                                    }
                                    intCumulateTask = left2;
                                    if (intCumulateTask == null) {
                                        break Label_0756;
                                    }
                                    break Label_0322;
                                }
                                continue;
                            }
                        }
                    }
                    if (o != null) {
                        o.fork();
                    }
                    else {
                        continue;
                    }
                }
                else {
                    int pendingCount;
                    while (((pendingCount = intCumulateTask.getPendingCount()) & 0x4) == 0x0) {
                        int n = ((pendingCount & 0x1) != 0x0) ? 4 : ((lo > origin) ? 2 : 6);
                        if (!intCumulateTask.compareAndSetPendingCount(pendingCount, pendingCount | n)) {
                            continue;
                        }
                        int out;
                        if (n != 2) {
                            int n2;
                            if (lo == origin) {
                                out = array[origin];
                                n2 = origin + 1;
                            }
                            else {
                                out = intCumulateTask.in;
                                n2 = lo;
                            }
                            for (int j = n2; j < hi; ++j) {
                                out = (array[j] = function.applyAsInt(out, array[j]));
                            }
                        }
                        else if (hi < fence) {
                            out = array[lo];
                            for (int i = lo + 1; i < hi; ++i) {
                                out = function.applyAsInt(out, array[i]);
                            }
                        }
                        else {
                            out = intCumulateTask.in;
                        }
                        intCumulateTask.out = out;
                        ArrayPrefixHelpers.IntCumulateTask intCumulateTask2;
                        while ((intCumulateTask2 = (ArrayPrefixHelpers.IntCumulateTask)intCumulateTask.getCompleter()) != null) {
                            int pendingCount2 = intCumulateTask2.getPendingCount();
                            if ((pendingCount2 & n & 0x4) != 0x0) {
                                intCumulateTask = intCumulateTask2;
                            }
                            else if ((pendingCount2 & n & 0x2) != 0x0) {
                                ArrayPrefixHelpers.IntCumulateTask left;
                                ArrayPrefixHelpers.IntCumulateTask right;
                                if ((left = intCumulateTask2.left) != null && (right = intCumulateTask2.right) != null) {
                                    int out2 = left.out;
                                    intCumulateTask2.out = ((right.hi == fence) ? out2 : function.applyAsInt(out2, right.out));
                                }
                                int n3 = (pendingCount2 & 0x1) == 0x0 && intCumulateTask2.lo == origin;
                                int count;
                                if ((count = (pendingCount2 | n | n3)) == pendingCount2 || intCumulateTask2.compareAndSetPendingCount(pendingCount2, count)) {
                                    n = 2;
                                    intCumulateTask = intCumulateTask2;
                                    if (n3 != 0) {
                                        intCumulateTask2.fork();
                                    }
                                    else {
                                        continue;
                                    }
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                if (intCumulateTask2.compareAndSetPendingCount(pendingCount2, pendingCount2 | n)) {
                                    break Label_0756;
                                }
                                continue;
                            }
                        }
                        if ((n & 0x4) != 0x0) {
                            intCumulateTask.quietlyComplete();
                            break;
                        }
                        break;
                    }
                    break;
                }
            }
            return;
        }
    }
}
